#!/usr/bin/perl

use strict;
use CPAN;

die "usage: $0 <core-file|pid> [/path/to/asterisk]\n" unless @ARGV;

my ($core, $asterisk) = @ARGV;
my $log = "$core.gdb.log";
rename($log, $log . "." . time());
$asterisk ||= "/usr/sbin/asterisk";

# do we have gdb?
if (!`gdb -v 2>>/dev/null`)
{
	print STDERR "GDB not found! Installing...\n";
	system("yum -y install gdb");
	print STDERR "\n";
}

# do we have Devel::GDB?
my $ver = eval "use Devel::GDB; \$Devel::GDB::VERSION";
if (!$ver)
{
	print STDERR "Devel::GDB not found! Installing...\n";
	CPAN::Shell->force("install", "Devel::GDB");
	print STDERR "\n";
}
eval("use Devel::GDB");

my $data;
my $gdb = new Devel::GDB;

l(localtime() . "\n");

g("file $asterisk");

# if we have a pid, dump a core and exit so we can restart and get calls.
# we can inspect the core file afterwards
if ($core =~ /^\d+$/)
{
	g("attach $core");
	g("generate-core-file");

	print STDERR "\nCore file dumped. You may now restart Asterisk if necessary.\n";
	print STDERR "When ready, rerun this program with the core file to inspect (this will NOT affect Asterisk).\n"; 
	$gdb->end();

	# sigcont - make sure asterisk continues to run when we're done
	kill(18, $core);
	exit;
}
g("core-file $core");
g("info thread");
g("thread apply all bt");

my ($thread, $tid, $frame, $tdata);
foreach (split(/\n/,  $data))
{
	if ($frame && /^Thread/)
	{
		last;
	}
	elsif (/Thread (\d+).*(?:process|LWP) (\d+)/)
	{
		($thread, $tid) = ($1, $2);
		$tdata = "";
	}
	elsif (!$frame && /^#(\d+) .* __ast_pthread_mutex_lock/)
	{
		$frame = $1;
	}

	$tdata .= "$_\n";
}

open M, "</var/log/asterisk/messages";
while (<M>)
{
	l($_) if /\[$tid\]/;
}
close M;

g("thread $thread");
my $f = g("frame $frame");

if ($f =~ /lock\(&?([^,\)]+)/)
{
	my $lock = $1;
	$lock =~ s/->mutex//;

	g("p $lock->thread");
	g("p $lock->file");
	g("p $lock->func");
	g("p $lock->lineno");
}

l("\nDeadlocked thread $thread (thread ID $tid, frame $frame)\n\n$tdata\n");

$gdb->end();
exit;

sub g
{
	my $d = "(gdb) $_[0]\n" . $gdb->get($_[0]);
	$data .= $d;
	l($d);
	return $d;
}

sub l
{
	print $_[0];

	open L, ">>$log";
	print L $_[0];
	close L;
}
