#!/usr/bin/perl

use Getopt::Long;
use Data::Dumper;
use strict;
use File::Copy;

# generic ip functions here:
require "/var/adm/bin/ip_funcs.pl";

my %setting;
my $mode;

if ( @ARGV > 1 )
{
		my($ip,$sn,$gw,$nw,$bc,$d1,$d2,$dns,$dhcpd2);
		GetOptions( 
			'mode=s' => \$mode,
			'ip=s' => \$ip,
			'sn=s' => \$sn,
			'gw=s' => \$gw,
			'nw=s' => \$nw,
			'bc=s' => \$bc,
			'd1=s' => \$d1,
			'd2=s' => \$d2,
			'dns=s' => \$dns,
			    'dhcpd2=s' => \$dhcpd2,
		);

		%setting = (
			ip => $ip,
			sn => $sn,
			gw => $gw,
			nw => $nw,
			bc => $bc,
			d1 => $d1,
			d2 => $d2,
			dns => $dns,
			    dhcpd2 => $dhcpd2,
			    );
}

else
{
    my $file = shift @ARGV;
    open my $fh, $file, or die $!;
    while(<$fh>)
    {
	chomp;
	my( $k,$v ) = split /\s*=\s*/,$_,2;
	$setting{$k} = $v;
    }
    
    $mode = $setting{mode};
    close $fh;    
}

$setting{eth} ||= 'eth0';
$setting{ip} ||= _get_ip_address($setting{eth});
$setting{sn} ||= _get_netmask($setting{eth});  
$setting{gw} ||= _get_default_route();
$setting{nw} ||= _get_network_addr($setting{ip},$setting{eth});
$setting{bc} ||= _get_broadcast_addr($setting{ip},$setting{eth});
my ($ns) = _get_nameservers();
$setting{d1} ||= $ns->[0];
$setting{d2} ||= $ns->[1];


unlink( "/tmp/ip_restored" );

ipset( $mode, \%setting );

exit(0);

##############################################################################
# ipset
##############################################################################
sub ipset
{
	my ($mode, $setting) = @_;

	# Do the config stuff in one place so vtuns are started/stopped
	# correctly
	net_stop();

	if ($mode eq 'STATIC')
	{
	    enable_static_ip($setting || {});
	    net_start();
	}
	else
	{
		enable_dhcp_client($setting);
	}


}
