#!/usr/bin/perl
# Copyright 2007 Fonality, Inc. ALL RIGHTS RESERVED

use Digest::MD5 qw/md5_hex/;
use Sys::Hostname;
use LWP::Simple;
use IO::Handle;
use IO::Socket;
use Socket;
#use IO;
#use IO::Seekable;
use strict;

# if this is a dell system we should NOT run
#if (-e '/etc/dell')
#{
#	exit(1);
#}

my %devices = map { $_ => 1 } qw/
    0004F2
    00E075
    00085D
    000590
    000413
    001243
    0003E3
    001A2F
    000B82
/;
if (open(DEVS, "</etc/asterisk/devices"))
{
    while (<DEVS>)
    {
        my @data = split(/,/);
        $devices{$data[0]} = $data[1];
    }
    close(DEVS);
}

$|++;
my $debug = $ARGV[0] eq "-D" ? 1 : 0;
my(%macs, $user, $pass);

my ($node) = hostname;
my $local_ip = inet_ntoa(scalar gethostbyname($node || 'localhost'));
$node =~ s/(?:trixbox|pbxtra)//;

open(VPN, "</etc/sysconfig/vpn");
my $stype = 'pbxtra';
while (<VPN>)
{
    if (/^GW.*trixbox/)
    {
        $stype = 'trixbox';
        last;                                                                                         
    }                                                                                                 
}                                                                                                     
close(VPN);

open CONF, "</etc/asterisk/fonality/globals.conf" || dprint("Can't open globals.conf for reading: $!", 1);
while (<CONF>)
{
    if (/IAXINFO\d?=(s\d{4,6}):([^;\s]+)/)
    {
        ($user, $pass) = ($1, $2);
        $user =~ s/[xi]$//;

        last if $user =~ /^s\d+$/;
    }
}
close(CONF);


unless ($debug)
{
	close(STDERR);
	close(STDOUT);
}

# monitor dhcp requests that aren't coming to the 2nd nic
fork() || monitor_packets();

# monitor tftp log
# ie: Serving 00085D1824DD.cfg to 10.10.10.164:1027
fork() || tail('tftp', '/var/log/tftp.log', 'Serving (?:cfg([a-fA-F\d]{12}))|(?:([a-fA-F\d]{12})\.cfg) to (\d+\.\d+\.\d+\.\d+)\:');

# monitor ftp log
fork() || tail('ftp', '/var/log/xferlog', '\d+\.\d+\.\d+\.\d+\s+\d+\s+/([A-Fa-f\d]{12})[^A-Fa-f\d]');

# monitor messages for dhcp requests on 2nd nic
fork() || tail('dhcpd', '/var/log/messages', 'dhcpd: DHCPREQUEST for .* from (\S+)');

exit;

# tail a file
sub tail
{
	my ($name, $file, $parse) = @_;

	dprint("Opening $file for reading");
	open(MSG, "<$file") || dprint("Can't read $file: $!", 1);

	# go to end of file
	seek(MSG, 0, 2);
	while (1)
	{
		my ($cursize) = (stat($file))[7];
		# if the file is now bigger than before
		if ($cursize > tell(MSG))
		{
			seek(MSG, tell(MSG), 0);
		}

		# if the file is smaller, it was truncated/rotated
		elsif ($cursize < tell(MSG))
		{
			close(MSG);
			open(MSG, "<$file") || dprint("Can't read $file: $!", 1);
		}

		while (<MSG>)
		{
			if (/$parse/)
			{
				my ($mac) = uc($1);
				$mac =~ s/://g;
				my $ip = ($2);
				dprint("($name) Request from $mac ($ip)\n");

				%macs = get_macs();
				if (!$macs{$mac})
				{
				    add_device($mac,$ip);
				}
			}
		}
		sleep(1);
	}
}

    
sub dprint
{
	my $log = shift;
	chomp($log);
	$log .= "\n";

	open(LOG, ">>/var/log/monitor_devices.log");
	print LOG localtime() . ": $$: $log";
	close(LOG);

	print STDERR $_[0] if $debug;

	if ($_[0])
	{
		die $log;
	}
}

sub monitor_packets
{
	# 11:17:43.693505 IP 10.10.10.244.bootpc > 255.255.255.255.bootps: BOOTP/DHCP, Request from 00:16:17:e3:fd:03, length: 300
	my $tcp = "/usr/sbin/tcpdump -lnp -i eth0 port bootpc";
	dprint("Running $tcp");
	open(TCPDUMP, "$tcp 2>>/dev/null|") || dprint("Can't run tcpdump: $!", 1);
	while (<TCPDUMP>)
	{
		if (/Request from ([\w\:]+)/)
		{
			my ($mac) = uc($1);
			$mac =~ s/://g;
			dprint("(tcpdump) Request from $mac\n");

			%macs = get_macs();
			if (!$macs{$mac})
			{
				add_device($mac);
			}
		}
	}
	close(TCPDUMP);
	exit;
}

sub add_device
{
	if ($devices{substr($_[0], 0, 6)})
	{
		# if it's a polycom phone that's being auto-provisioned
		# we want to find a description for it
		my $desc;
		if(substr($_[0],0,6) =~ /0004F2|00E075/i)
		{
		    $desc = search_bootlog($_[0]);
		    if ($desc)
		    {
			dprint("Phone model found: $desc\n");
		    }
		    # if we can't find a description, it probably means
		    # we need to wait longer 
		    else
		    {
			return 0;
		    }
		}

		dprint("Adding $_[0]\n");
		$macs{$_[0]} = 1;

		my $resp = get("http://cp." . ($stype eq 'trixbox' ? 'trixbox' : 'fonality') . ".com/add_devices.cgi?server_id=$node&device=$_[0]&description=$desc&hash=" . md5_hex($_[0] . $pass));
		$resp =~ s/\n/ /g;
		print "$resp\n";
	}
}

sub search_bootlog
{
    my $mac = lc(shift);
    my $file = "/tftpboot/$mac-boot.log";
    my $desc;

    # search through the bootlog to find the phone model
    open(BOOTLOG, $file) || dprint("Can't read $file: $!");
    while(<BOOTLOG>)
    {
	# matches:
	# Platform: Model=SoundPoint IP 650, Assembly=2345-12600-001 Rev=B
	if(/Platform\: Model=\D+(\d+)\D/)
	{
	    $desc = $1;
	    last;
	}
    }
    close BOOTLOG;

    # description becomes: Auto-detected NNN
    
    return $desc;
}


sub get_macs
{
	# scan for existing devices
	my %macs = ();
	open(SIP, "</etc/asterisk/sip.conf") || dprint("Can't read sip.conf: $!", 1);
	while (<SIP>)
	{
		if (/^\[(.*?)\]/)
		{
			$macs{$1} = 1;
		}
	}
	close(SIP);
	return %macs;
}
