#!/usr/bin/perl
# Copyright 2006 Fonality, Inc. ALL RIGHTS RESERVED
#
# This script needs to be executed X times every hour on every
# pbxtra machine. This script connects to the primary server,
# sends authentication information and exits. It is used so
# the primary server knows the public IP address of this host.

use strict;
use LWP::Simple;
use Digest::MD5 qw/md5_hex/;
use Getopt::Long;
use Sys::Hostname;
use File::Copy;

close(STDERR);
close(STDOUT);
open(STDERR, ">>/var/log/ping_main.log");
open(STDOUT, ">>/var/log/ping_main.log");
print localtime() . "\n";

my ($user, $bup_user, $hash, $randsleep);

GetOptions ("randsleep"  => \$randsleep);

if($randsleep) { # Introduce an arbitrary delay to avoid stampeedes
	srand($$ + time); # Seed the random function
	sleep(int(rand(300)));
}

# For Live Backup Servers, don't try to update when user does not match hostname
my ($node) = hostname;
$node =~ s/(?:trixbox|pbxtra)/s/;

open(VPN, "</etc/sysconfig/vpn");
my $stype = 'pbxtra';
while (<VPN>)
{
	if (/^GW.*trixbox/)
	{
		if (-e '/etc/dell/')
		{
			# We don't yet have these domain names set up, so for now go to trixbox
			$stype = 'trixbox';
			#$stype = 'pbx';
		}
		else
		{
			$stype = 'trixbox';
		}
		last;
	}
}
close(VPN);

# get the backup server user
# the entries in primary or standby look like: s111111.trixbox.fonality.com; we just want the 's111111' part
my $peer_file = -e '/etc/asterisk/standby' ? '/etc/asterisk/standby' : -e '/etc/standby' ? '/etc/standby' : '';
if ($peer_file && open my $standby, '<', $peer_file)
{
	chomp($bup_user = <$standby>);
	close $standby;
	$bup_user =~ s/(s\d+).*/$1/;
}
else
{
	$bup_user = '';
}

open CONF, "</etc/asterisk/fonality/globals.conf" || die "Can't open globals.conf for reading: $!";
while (<CONF>)
{
	if (/^IAXINFO/)
	{
		if (/^IAXINFO\d?=(s\d{4,6}):([^;\s]+)/)
		{
			($user, $hash) = ($1, md5_hex($2));
			$user =~ s/[xi]$//;
		}

		# Even if we didn't find the right credentials, we found the line they should be on.
		# If they aren't here, no use in checking the rest of the file.
		last;
	}
}
close(CONF);

# Replace the user with the node, for use with LBS.
if ($node =~ /^s\d+$/)
{
	$user = $node;
}

# should we get the ip from a specific interface?
my $interface;
if (open(INTERFACE, "</etc/asterisk/interface"))
{
	chomp($interface = <INTERFACE>);
	close(INTERFACE);
}

# get the internal ip
my $internal;
my $eth0;
my $int;    
foreach (`/sbin/ifconfig -a`)
{
	# we found an interface
	if (/^(\S+)/)
	{
		$int = $1;
	}

	# we found our specified interface
	elsif ($int eq $interface && /inet (?:addr:)?(\d\S+)/)
	{
		$internal = $1;
	}

	# support both linux and *bsd ifconfig outputs
	elsif ($int eq "eth0" && /inet (?:addr:)?(\d\S+)/)
	{
		$eth0 = $1;
	}
}

# set the internal address to eth0's if we don't have a specific one
$internal ||= $eth0;

# Get the amount of ram in the system
#
# We get the amount of ram in your system to store locally. By storing it
# locally it enables to speed up the options->settings page. If you have a 
# machine with less then 512MB of ram in it you will get a warning when
# enabling hud. 
#
# This page used to poll your server when loading the page for this information 
# causing the page to load slowly, and put load on your machine.
#
open(MEMINFO, '</proc/meminfo');
my @data = <MEMINFO>;
close(MEMINFO);

my $memory;

foreach my $line (@data)
{
        if ($line =~ /MemTotal:\s*(\d+)/)
        {
            # get # of kb
            $memory = $1;
        }
}


#######################################
#
# begin named portion
#
#


# some globals we need
my $RELOAD_NAMED   = "/etc/init.d/named reload";
my $BOOT_NAMED     = "/sbin/chkconfig --level 2345 named on";
my $START_NAMED    = "/etc/init.d/named start";
my $NAMED_CONF_1   = "/etc/named.conf";
my $NAMED_CONF_2   = "/var/named/chroot/etc/named.conf";
my $DB             = "/var/named/chroot/var/named/db.$user.$stype.fonality.com";
my $BUP_DB         = "/var/named/chroot/var/named/db.$bup_user.$stype.fonality.com";
my $DB_SYMLINK     = "/var/named/db.$user.$stype.fonality.com"; 
my $BUP_DB_SYMLINK = "/var/named/db.$bup_user.$stype.fonality.com";
my $CHROOT_DIR_1   = "/var/named/chroot/var/named";
my $CHROOT_DIR_2   = "/var/named/chroot/etc";

# should we reload?
my $reload = 0;

# make sure we have a /var/named/chroot
create_dir($CHROOT_DIR_1);
create_dir($CHROOT_DIR_2);

# make sure named starts on bootup
if (`/sbin/chkconfig --list named` =~ /[2345]:off/)
{
	system($BOOT_NAMED);
}

# make sure named is running
if (`/etc/init.d/named status` !~ /server is up/)
{
	system($START_NAMED);
}

# check for a zone file
unless (check_zone($user, $internal, $DB))
{
	create_zone($user, $internal, $DB);
	$reload++;
}
if ($bup_user && !check_zone($bup_user, $internal, $BUP_DB))
{
	create_zone($bup_user, $internal, $BUP_DB);
	$reload++;
}

# check for symlink to zone file
unless (check_zone($user, $internal, $DB_SYMLINK))
{
	create_symlink_to_zone($DB, $DB_SYMLINK);
	$reload++;
}
if ($bup_user && !check_zone($bup_user, $internal, $BUP_DB_SYMLINK))
{
	create_symlink_to_zone($BUP_DB, $BUP_DB_SYMLINK);
	$reload++;
}

# check entry in named.conf
unless (check_named_conf($user, $NAMED_CONF_1))
{
	create_named_conf($user, $NAMED_CONF_1);
	$reload++;
}
if ($bup_user && !check_named_conf($bup_user, $NAMED_CONF_1))
{
	create_named_conf($bup_user, $NAMED_CONF_1);
	$reload++;
}

# check entry in named.conf
unless (check_named_conf($user, $NAMED_CONF_2))
{
	create_named_conf($user, $NAMED_CONF_2);
	$reload++;
}
if ($bup_user && !check_named_conf($bup_user, $NAMED_CONF_2))
{
	create_named_conf($bup_user, $NAMED_CONF_2);
	$reload++;
}


# should we reload?
if ($reload)
{
	system($RELOAD_NAMED);
}

# In order to handle the case where we switch between being primary to backup after an outage
# pick up the new primary server from the ping and make sure the files are correct when doing so
my $rhost = ($stype eq 'pbxtra' ? 'update.fonality.com' : 'iptrack.trixbox.com');
my $ret = get("http://$rhost/ping.cgi?server_id=$user&hash=$hash&internal=$internal&memory=$memory");
chomp($ret);
if ($ret =~ /^\d{4,6}$/)
{
	if (-e '/etc/asterisk/standby' || -e '/etc/standby')
	{
		if (-e '/etc/asterisk/standby')
		{
			rename '/etc/asterisk/standby', '/etc/asterisk/primary';
		}
		else
		{
			rename '/etc/standby', '/etc/asterisk/primary';
		}

		if (readlink '/etc/standby')
		{
			unlink '/etc/standby';
		}
	}

	if (!-e '/etc/asterisk/primary')
	{
		# In case it's 'pbx' for dell, make sure it still goes to the right place
		my $tmp = $stype eq 'pbxtra' ? 'pbxtra' : 'trixbox';

		open my $primary, '>', '/etc/asterisk/primary';
		print $primary "s${ret}b.$tmp.fonality.com";
		close $primary;
		chown 99, 99, '/etc/asterisk/primary';
	}
}

# Support auto-provisioning of Polycoms on firmware 2.2
if (`/sbin/chkconfig --list dhcpd` !~ /on/)
{
    # overwite /etc/dhcpd.conf with tbp-cp1:/root/dhcpd.poly.conf
    my $tmp_file = "/tmp/dhcpd.".time()."conf";
    open TMP, ">$tmp_file" || die "could not open $tmp_file for writing: $!";    
    
    while(<DATA>)
    {
		# switch ip address in that file to $internal ip.
		s/(\d+\.\d+\.\d+\.\d+)\"/$internal\"/;

		print TMP "$_";
    }
    close TMP;
    
    copy($tmp_file, '/etc/dhcpd.conf');
    

    `/sbin/chkconfig --level 2345 dhcpd on`;

    `/etc/init.d/dhcpd start`;

    unlink($tmp_file);
}
elsif (`/sbin/chkconfig --list dhcpd` =~ /on/)
{
    my $old_file = '/etc/dhcpd.conf';

    # check if /etc/dhcpd.conf already has DATA in it
    my $data = 'subclass "phone" 00:04:f2';
	
    my $matched_lines = 0;
    my $change_ip = 0;
    my $subnet_lines;
	
    my $tmp_file = "/tmp/dhcpd.".time()."conf";
    open TMP, ">$tmp_file" || die "could not open $tmp_file for writing: $!";
    
    open OLD, "$old_file" || die ("Could not open $old_file: $!");

    while(<OLD>)
    {
		if (/$data/)
		{
		    $matched_lines++;
		}
		if ($matched_lines)
		{
		    # if it DATA is already in dhcpd.conf, check for right IP address
		    if (/(\d+\.\d+\.\d+\.\d+)\"/)
		    {
				if ($1 ne $internal)
				{
					s/(\d+\.\d+\.\d+\.\d+)/$internal/;
					$change_ip = 1;
				}
			}
			elsif (/\Qoption tftp-server-name ""\E/)
			{
				s/\Q""\E/"$internal"/;
				$change_ip = 1;
			}
	    
		    # if it has 'tftp://', remove it. We don't want to confuse
		    # polycom, which doesn't like tftp apparently
		    # ie: "tftp://192.168.60.6"
		    s/tftp\:\/\///;
		}
		# if the 2nd network card is dhcp :. subnet is not 0s
		if (/^subnet/ && !/^subnet\s+0\.0\.0\.0/)
		{
		    # save all the subnet stuff for appending later
		    $subnet_lines .= "$_";
		    # grab all the subnet lines (inside {}'s)
		    while(<OLD>)
		    {
				$subnet_lines .= "$_";
				if (/^}/)
				{
					$_ = <OLD>;
					last;
				}
			}
		}
	 	print TMP "$_";
    }

    # if no DATA in it, append DATA to dhcpd.conf
    unless ($matched_lines)
    {
		while(<DATA>)
		{
	    	# switch ip address in that file to $internal ip.
	    	s/(192\.168\.0\.198)/$internal/;

	    	print TMP "$_";
		}
    }

    #append the subnet lines after the subnet 0.0.0.0 stuff
    print TMP $subnet_lines;	    

    close TMP;
    
    copy($tmp_file, '/etc/dhcpd.conf');
    unlink($tmp_file);
    
    # if changed IP or appended DATA run /etc/init.d/dhcpd
    if ($change_ip || !$matched_lines)
    {
		`/etc/init.d/dhcpd start`;
    }
}

exit;

# check to see if a zone file exists / whether it needs a new IP
sub check_zone
{
	my ($user, $ip, $PATH) = @_;

	# check to see if the zone file exists and has the correct ip
	open(ZONE, "<$PATH") || return 0;
	while (<ZONE>)
	{
		# search for an 'A' record pointing to the IP
		# or find a 'DONT_CHANGE' line meaning we should not change anything
		if (/^\s*A\s*$ip\D|DONT_CHANGE/)
		{
			close(ZONE);
			return 1;
		}
	}
	close(ZONE);

	return 0;
}

# check to see if named.conf properly has the zone
sub check_named_conf
{
	my ($user, $CONF) = @_;

	# check to see if the zone file exists and has the correct ip
	open(CONF, "<$CONF") || return 0;
	while (<CONF>)
	{
		# search for the proper zone
		# or find a 'DONT_CHANGE' line meaning we should not change anything
		if (/^\s*zone\s+"$user\.$stype\.fonality\.com"|DONT_CHANGE/)
		{
			close(CONF);
			return 1;
		}
	}
	close(CONF);

	return 0;
}


# produce the zone file
sub create_zone
{
	my ($user, $ip, $db) = @_;

	# create a serial based off of the time
	my @time = localtime();
	my $serial = sprintf("%04d%02d%02d%02d", $time[5] + 1900, $time[4] + 1, $time[3], $time[2]);

	# create the zone file
	open(ZONE, '>', $db) || return 0;
	print ZONE << "EOF";
\$ORIGIN .
\$TTL 3600	; 1 hour
$user.$stype.fonality.com	IN SOA	$user.$stype.fonality.com. support.fonality.com. (
		$serial	; serial
		10800		; refresh (3 hours)
		1800		; retry (30 minutes)
		604800		; expire (1 week)
		3600		; minimum (1 hour)
		)
		NS		$user.$stype.fonality.com.
		A		$ip
		MX	10	$user.$stype.fonality.com.
EOF
	close(ZONE);

	return 1;
}


# add the zone to named.conf
sub create_named_conf
{
	my ($user, $CONF) = @_;

	# read in the real conf
	open(CONF, "<$CONF");
	my @data = <CONF>;
	close(CONF);

	# let's add the zone to the end of the file, or before the first include
	my $i;
	for ($i = 0; $i < @data; $i++)
	{
		if ($data[$i] =~ /^include/)
		{
			last;
		}
	}

	splice(@data, $i, 0, << "EOF");

zone "$user.$stype.fonality.com" IN {
	type master;
	file "db.$user.$stype.fonality.com";
	allow-update { none; };
};

EOF

	# create a temp zone file
	my $tmp = $CONF . "." . time();
	open(CONF, ">$tmp") || return 0;
	print CONF @data;
	close(CONF);

	# move the new conf over to the real location
	rename($tmp, $CONF);

	return 1;
}


# create the zone symlink
sub create_symlink_to_zone
{
	my ($db, $sym) = @_;

	# backup the file as it may exist
	rename($sym, $sym . "." . time());

	# symlink the real zone to a place where it _might_ be read in the future
	symlink($db, $sym);

	return 1;
}

# create a directory if it doesn't exist
sub create_dir
{
	my ($DIR) = @_;

	unless (-d $DIR)
	{
		my $tmpdir;
		foreach my $dir (split(/\//, $DIR))
		{
			$tmpdir .= "/$dir";

			# make the directory unless it exists
			unless (-d $tmpdir)
			{
				mkdir($tmpdir);
			}
		}
	}
}


__DATA__
ddns-update-style none;

class "phone" {
    match substring(hardware,1,3);
}

# Polycom
subclass "phone" 00:04:f2;
subclass "phone" 00:90:7a;
subclass "phone" 00:e0:75;

# Aastra
subclass "phone" 00:08:5d;
subclass "phone" 00:10:bc;

subnet 0.0.0.0 netmask 0.0.0.0 {
    authoritative;
    option tftp-server-name "192.168.0.198";
}
