#!/usr/bin/perl
#
# Copyright 2007, Fonality Inc. ALL RIGHTS RESERVED.

my $width = 550; # 850
my $height = 260; # 230
# nice ourselves to 5
system("/usr/bin/renice 5 $$ >>/dev/null 2>>/dev/null");

use lib qw( /usr/local/rrdtool-1.2.13/lib/perl );
use lib qw( /usr/local/rrdtool-1.2.15/lib/perl );
use RRDs;
use strict;

my @types = qw/server call network/;
my %DB = (
	server => "/var/lib/asterisk/server_stats.rrd",
	call => "/var/lib/asterisk/call_stats.rrd",
	network => "/var/lib/asterisk/network_stats.rrd",
);

my $end = time();
my $start = $end - (60 * ($ARGV[0] || 5));

my $DIR = "/usr/local/apache/htdocs/graphs";
mkdir($DIR);


# set our times properly
my @time = localtime($start);
my @ntime = localtime($end);

my $day = sprintf("%04d-%02d-%02d", $time[5]+1900, $time[4]+1, $time[3]);
my $nday = sprintf("%02d-%02d-%04d", $time[4]+1, $time[3], $time[5]+1900);
$DIR .= "/$day";
mkdir($DIR);

my $file = sprintf("%02d_%02d", $ntime[2], $ntime[1]);

$time[1]  = sprintf("%02d", $time[1]);
$ntime[1] = sprintf("%02d", $ntime[1]);

my ($am, $nam) = ("am", "am");
if ($time[2] > 12 || $time[2] == 0)
{
	$am = "pm" if $time[2];
	$time[2] = abs($time[2] - 12);
}
elsif ($time[2] == 12)
{
	$am = "pm";
}

if ($ntime[2] > 12 || $ntime[2] == 0)
{
	$nam = "pm" if $ntime[2];
	$ntime[2] = abs($ntime[2] - 12);
}
elsif ($ntime[2] == 12)
{
	$nam = "pm";
}

my $type = "server";
RRDs::graph "$DIR/$type-$file.png",
	  "--title", "Resources from $time[2]:$time[1] $am to $ntime[2]:$ntime[1] $nam on $nday",
	  "--start", $start,
	  "--end", $end,
	  "--lower-limit=0",
	  "--upper-limit=100",
	  "--interlace",
	  "--imgformat","PNG",
	  "--width=$width",
	  "--height=$height",
          "--x-grid=SECOND:10:MINUTE:1:MINUTE:1:0:%l:%M %P",
	  "DEF:load=$DB{$type}:load:AVERAGE",
	  "DEF:mem=$DB{$type}:mem:AVERAGE",
	  "DEF:hdd=$DB{$type}:hdd:AVERAGE",
	  "DEF:ast=$DB{$type}:ast:AVERAGE",
	  "DEF:hud=$DB{$type}:hud:AVERAGE",
	  "LINE2:load#2AC627:System CPU",
	  "LINE2:mem#FF0000:Memory Swapped",
	  "LINE2:hdd#3300FF:Hard Drive",
	  "LINE2:ast#EDA01A:PBXtra RAM",
	  "LINE2:hud#FA60FA:HUD CPU",
	;


$type = "call";
RRDs::graph "$DIR/$type-$file.png",
	  "--title", "Calls from $time[2]:$time[1] $am to $ntime[2]:$ntime[1] $nam on $nday",
	  "--start", $start,
	  "--end", $end,
	  "--lower-limit=0",
	  "--interlace",
	  "--imgformat","PNG",
	  "--width=$width",
	  "--height=$height",
          "--x-grid=SECOND:10:MINUTE:1:MINUTE:1:0:%l:%M %P",
	  "DEF:calls=$DB{$type}:calls:AVERAGE",
	  "DEF:conf=$DB{$type}:conf:AVERAGE",
	  "DEF:rec=$DB{$type}:rec:AVERAGE",
	  "DEF:trunks=$DB{$type}:trunks:AVERAGE",
	  "LINE3:calls#2AC627:Channels",
	  "LINE3:conf#FF0000:Conferences",
	  "LINE3:rec#3300FF:Recordings",
	  "LINE3:trunks#EDA01A:Trunks",
	;


$type = "network";
RRDs::graph "$DIR/$type-$file.png",
	  "--title", "Network from $time[2]:$time[1] $am to $ntime[2]:$ntime[1] $nam on $nday",
	  "--start", $start,
	  "--end", $end,
	  "--lower-limit=0",
	  "--interlace",
	  "--imgformat","PNG",
	  "--width=$width",
	  "--height=$height",
          "--x-grid=SECOND:10:MINUTE:1:MINUTE:1:0:%l:%M %P",
	  "DEF:rx=$DB{$type}:rx:AVERAGE",
	  "DEF:tx=$DB{$type}:tx:AVERAGE",
	  "LINE3:rx#2AC627:kbps received",
	  "LINE3:tx#FF0000:kbps sent",
	;
print RRDs::error;

=pod
                $DB, "--step=1",
                "DS:load:GAUGE:2:0:100",
                "DS:mem:GAUGE:2:0:100",
                "DS:hdd:GAUGE:2:0:100",
                "DS:ast:GAUGE:2:0:100",
                "DS:hud:GAUGE:2:0:100",
                "DS:time:GAUGE:2:0:9999999999",
                "RRA:$dst:0.5:1:3600",
                "RRA:$dst:0.5:360:60",
                "RRA:$dst:0.5:1440:60",
                "RRA:$dst:0.5:10080:60",
                "RRA:$dst:0.5:43200:60"

                RRDs::create(
                $DB2, "--step=1",
                "--start=$start",
                "DS:calls:GAUGE:2:0:1536",
                "DS:conf:GAUGE:2:0:1536",
                "DS:rec:GAUGE:2:0:1536",
                "DS:trunks:GAUGE:2:0:1536",
                "DS:time:GAUGE:2:1154051743:9154051743",
                "RRA:$dst:0.5:1:3600",
                "RRA:$dst:0.5:360:60",
                "RRA:$dst:0.5:1440:60",
                "RRA:$dst:0.5:10080:60",
                "RRA:$dst:0.5:43200:60"
#print RRDs::error;
#print scalar(@{$data});

