#!/usr/bin/perl
#
# Transcode MP3s/voice prompts to other formats
# to prevent transcoding during calls.
#
# usage: $0 <file1> [file2] [file3] [...]
#

use strict;
use Cwd;

my ($overwrite, @files) = usage();

my $path = cwd;
my $g729 = </usr/lib/asterisk/modules/codec_g729*.so> ? 1 : 0;
my $sox = "/usr/bin/sox";
my $mp3 = 0;
my $mpg123 = (
	-e "/usr/local/bin/mpg123" ? "/usr/local/bin/mpg123" :
		-e "/usr/local/bin/mpg123.real" ? "/usr/local/bin/mpg123.real" :
		undef
	);

foreach my $file (@files)
{
		if ($file !~ m|^/|)
		{
			$file = "$path/$file";
		}

		my $native;
		print "Transcoding $file...\n";

		# if it's an mp3, get our raw/wav file out of it
		if ($file =~ s/\.(mp3)$//i)
		{
			if (!$mpg123)
			{
				print STDERR "No support for MP3 transcoding -- mpg123 not found!\n";
				next;
			}

			$mp3++;
			system "$mpg123 -s --rate 44100 --mono $file.$1 > $file.raw" if $overwrite || !-e "$file.raw";
			system "$sox -r 44100 -w -s -c 1 $file.raw -r 8000 -c 1 $file.wav" if $overwrite || !-e "$file.wav";

			# raw files are huge, delete it immediately
			unlink("$file.raw");

			$native = "wav";
		}
		elsif ($file =~ s/\.(gsm|wav)$//i)
		{
			$native = $1;
		}
		else
		{
			print STDERR "No support for transcoding file type $file!\n";
			next;
		}

		if ($overwrite || !-e "$file.wav")
		{
			system "$sox $file.$native -r 8000 -s -w -c 1 $file.wav" unless $native eq "wav";
		}
		if ($overwrite || !-e "$file.gsm")
		{
			system "$sox $file.$native -t gsm -r 8000 -b -c 1 $file.gsm" unless $native eq "gsm";
		}
		if ($overwrite || !-e "$file.ulaw")
		{
			system "$sox $file.$native -t ul -r 8000 -b -c 1 $file.ulaw" unless $native eq "ulaw";
		}
		if ($overwrite || !-e "$file.alaw")
		{
			system "$sox $file.$native -t al -r 8000 -b -c 1 $file.alaw" unless $native eq "alaw";
		}
#		if ($overwrite || !-e "$file.sln")
#		{
#			system "$sox $file.$native -t raw -r 8000 -s -w -c 1 $file.sln" unless $native eq "sln";
#		}

		# g729 conversion only if we have a codec
		if ($g729)
		{
			system "/usr/sbin/asterisk -rx 'file convert $file.$native $file.g729'";
		}
}

# reload to read in our new moh files
if ($mp3)
{
	utime(time(), time(), "/etc/asterisk/musiconhold.conf");
	system "/usr/sbin/asterisk -rx 'reload res_musiconhold.so'";
}

exit;

sub usage
{
	die "usage: $0 [-d (don't overwrite)] <file1> [file2] [file3] [...]\n" unless @ARGV;

	my $overwrite = 1;
	if ($ARGV[0] eq "-d")
	{
		$overwrite = 0;
		shift(@ARGV);
	}

	return ($overwrite, @ARGV);
}
